@@ -31,22 +31,48 @@
 
 G_BEGIN_DECLS
 
+/**
+ * GI_IS_ENUM_INFO
+ * @info: an info structure
+ *
+ * Checks if @info is a #GIEnumInfo.
+ */
 #define GI_IS_ENUM_INFO(info) \
     ((g_base_info_get_type((GIBaseInfo*)info) ==  GI_INFO_TYPE_ENUM) || \
      (g_base_info_get_type((GIBaseInfo*)info) ==  GI_INFO_TYPE_FLAGS))
 
+/**
+ * GI_IS_VALUE_INFO
+ * @info: an info structure
+ *
+ * Checks if @info is a #GIValueInfo.
+ */
 #define GI_IS_VALUE_INFO(info) \
     (g_base_info_get_type((GIBaseInfo*)info) ==  GI_INFO_TYPE_VALUE)
 
+
+GI_AVAILABLE_IN_ALL
 gint           g_enum_info_get_n_values      (GIEnumInfo  *info);
+
+GI_AVAILABLE_IN_ALL
 GIValueInfo  * g_enum_info_get_value         (GIEnumInfo  *info,
 					      gint         n);
+
+GI_AVAILABLE_IN_ALL
 gint              g_enum_info_get_n_methods     (GIEnumInfo  *info);
+
+GI_AVAILABLE_IN_ALL
 GIFunctionInfo  * g_enum_info_get_method        (GIEnumInfo  *info,
 						 gint         n);
+
+GI_AVAILABLE_IN_ALL
 GITypeTag      g_enum_info_get_storage_type  (GIEnumInfo  *info);
+
+GI_AVAILABLE_IN_ALL
 const gchar *  g_enum_info_get_error_domain  (GIEnumInfo  *info);
 
+
+GI_AVAILABLE_IN_ALL
 gint64         g_value_info_get_value        (GIValueInfo *info);
 
 G_END_DECLS
