@@ -24,10 +24,8 @@ import operator
 from itertools import chain
 from collections import OrderedDict
 
-from . import message
-
 from .sourcescanner import CTYPE_TYPEDEF, CSYMBOL_TYPE_TYPEDEF
-from .message import Position
+from .message import Position, warn
 from .utils import to_underscores
 
 
@@ -233,6 +231,15 @@ TYPE_DOUBLE = Type(target_fundamental='gdouble', ctype='gdouble')
 TYPE_LONG_DOUBLE = Type(target_fundamental='long double',
                         ctype='long double')
 TYPE_UNICHAR = Type(target_fundamental='gunichar', ctype='gunichar')
+# Platform-specific types
+TYPE_TIME_T = Type(target_fundamental='time_t', ctype='time_t')
+TYPE_OFF_T = Type(target_fundamental='off_t', ctype='off_t')
+# Unix-specific types that are handled here for historical reasons
+TYPE_DEV_T = Type(target_fundamental='dev_t', ctype='dev_t')
+TYPE_GID_T = Type(target_fundamental='gid_t', ctype='gid_t')
+TYPE_PID_T = Type(target_fundamental='pid_t', ctype='pid_t')
+TYPE_SOCKLEN_T = Type(target_fundamental='socklen_t', ctype='socklen_t')
+TYPE_UID_T = Type(target_fundamental='uid_t', ctype='uid_t')
 
 # C types with semantics overlaid
 TYPE_GTYPE = Type(target_fundamental='GType', ctype='GType')
@@ -241,13 +248,40 @@ TYPE_FILENAME = Type(target_fundamental='filename', ctype='gchar*')
 
 TYPE_VALIST = Type(target_fundamental='va_list', ctype='va_list')
 
-BASIC_TYPES = [TYPE_BOOLEAN, TYPE_INT8, TYPE_UINT8, TYPE_INT16,
-               TYPE_UINT16, TYPE_INT32, TYPE_UINT32, TYPE_INT64,
-               TYPE_UINT64, TYPE_CHAR, TYPE_SHORT, TYPE_USHORT, TYPE_INT,
-               TYPE_UINT, TYPE_LONG, TYPE_ULONG, TYPE_SIZE, TYPE_SSIZE,
-               TYPE_LONG_LONG, TYPE_LONG_ULONG,
-               TYPE_FLOAT, TYPE_DOUBLE,
-               TYPE_LONG_DOUBLE, TYPE_UNICHAR, TYPE_GTYPE]
+BASIC_TYPES = [
+    TYPE_BOOLEAN,
+    TYPE_INT8,
+    TYPE_UINT8,
+    TYPE_INT16,
+    TYPE_UINT16,
+    TYPE_INT32,
+    TYPE_UINT32,
+    TYPE_INT64,
+    TYPE_UINT64,
+    TYPE_CHAR,
+    TYPE_SHORT,
+    TYPE_USHORT,
+    TYPE_INT,
+    TYPE_UINT,
+    TYPE_LONG,
+    TYPE_ULONG,
+    TYPE_SIZE,
+    TYPE_SSIZE,
+    TYPE_LONG_LONG,
+    TYPE_LONG_ULONG,
+    TYPE_TIME_T,
+    TYPE_OFF_T,
+    TYPE_FLOAT,
+    TYPE_DOUBLE,
+    TYPE_LONG_DOUBLE,
+    TYPE_UNICHAR,
+    TYPE_GTYPE,
+    TYPE_DEV_T,
+    TYPE_GID_T,
+    TYPE_PID_T,
+    TYPE_SOCKLEN_T,
+    TYPE_UID_T,
+]
 
 BASIC_GIR_TYPES = [TYPE_INTPTR, TYPE_UINTPTR]
 BASIC_GIR_TYPES.extend(BASIC_TYPES)
@@ -261,9 +295,19 @@ GIR_TYPES.extend([TYPE_STRING, TYPE_FILENAME, TYPE_VALIST])
 POINTER_TYPES = [TYPE_ANY, TYPE_INTPTR, TYPE_UINTPTR]
 
 INTROSPECTABLE_BASIC = list(GIR_TYPES)
-for v in [TYPE_NONE, TYPE_ANY,
-          TYPE_LONG_LONG, TYPE_LONG_ULONG,
-          TYPE_LONG_DOUBLE, TYPE_VALIST]:
+for v in [
+    TYPE_NONE,
+    TYPE_ANY,
+    TYPE_LONG_LONG,
+    TYPE_LONG_ULONG,
+    TYPE_LONG_DOUBLE,
+    TYPE_VALIST,
+    TYPE_DEV_T,
+    TYPE_GID_T,
+    TYPE_PID_T,
+    TYPE_SOCKLEN_T,
+    TYPE_UID_T,
+]:
     INTROSPECTABLE_BASIC.remove(v)
 
 type_names = {}
@@ -307,6 +351,20 @@ type_names['uint32_t'] = TYPE_UINT32
 type_names['int64_t'] = TYPE_INT64
 type_names['uint64_t'] = TYPE_UINT64
 
+type_names['__int8_t'] = TYPE_INT8
+type_names['__uint8_t'] = TYPE_UINT8
+type_names['__int16_t'] = TYPE_INT16
+type_names['__uint16_t'] = TYPE_UINT16
+type_names['__int32_t'] = TYPE_INT32
+type_names['__uint32_t'] = TYPE_UINT32
+type_names['__int64_t'] = TYPE_INT64
+type_names['__uint64_t'] = TYPE_UINT64
+type_names['__off_t'] = TYPE_OFF_T
+type_names['__gid_t'] = TYPE_GID_T
+type_names['__uid_t'] = TYPE_UID_T
+type_names['__pid_t'] = TYPE_PID_T
+type_names['__socklen_t'] = TYPE_SOCKLEN_T
+
 # A few additional GLib type aliases
 type_names['guchar'] = TYPE_UINT8
 type_names['gchararray'] = TYPE_STRING
@@ -346,13 +404,13 @@ type_names['size_t'] = type_names['gsize']
 type_names['ssize_t'] = type_names['gssize']
 type_names['uintptr_t'] = type_names['guintptr']
 type_names['intptr_t'] = type_names['gintptr']
-type_names['time_t'] = TYPE_LONG
-type_names['off_t'] = type_names['gsize']
-type_names['pid_t'] = TYPE_INT
-type_names['uid_t'] = TYPE_UINT
-type_names['gid_t'] = TYPE_UINT
-type_names['dev_t'] = TYPE_INT
-type_names['socklen_t'] = TYPE_INT32
+type_names['time_t'] = TYPE_TIME_T
+type_names['off_t'] = TYPE_OFF_T
+type_names['pid_t'] = TYPE_PID_T
+type_names['uid_t'] = TYPE_UID_T
+type_names['gid_t'] = TYPE_GID_T
+type_names['dev_t'] = TYPE_DEV_T
+type_names['socklen_t'] = TYPE_SOCKLEN_T
 
 # Obj-C
 type_names['id'] = TYPE_ANY
@@ -433,7 +491,7 @@ but adds it to things like ctypes, symbols, and type_names.
             self.type_names[node.gtype_name] = node
         elif isinstance(node, Function):
             self.symbols[node.symbol] = node
-        if isinstance(node, (Compound, Class, Interface, Boxed)):
+        if isinstance(node, (Compound, Class, Interface, Boxed, Pointer)):
             for fn in chain(node.methods, node.static_methods, node.constructors):
                 if not isinstance(fn, Function):
                     continue
@@ -686,6 +744,9 @@ class Callable(Node):
         self.throws = not not throws
         self.instance_parameter = None  # Parameter
         self.parent = None  # A Class or Interface
+        self.finish_func = None
+        self.sync_func = None
+        self.async_func = None
 
     def _get_retval(self):
         return self._retval
@@ -757,6 +818,7 @@ class Function(Callable):
         self.internal_skipped = False   # if True, this func will not be written to GIR
         self.set_property = None        # Property name
         self.get_property = None        # Property name
+        self.is_inline = False
 
     def clone(self):
         clone = copy.copy(self)
@@ -779,7 +841,7 @@ class Function(Callable):
         # Returns GType
         rettype = self.retval.type
         if (not rettype.is_equiv(TYPE_GTYPE) and rettype.target_giname != 'Gtk.Type'):
-            message.warn("function '%s' returns '%r', not a GType" % (self.name, rettype))
+            warn("function '%s' returns '%r', not a GType" % (self.name, rettype))
             return False
 
         return True
@@ -1197,6 +1259,32 @@ class Boxed(Node, Registered):
             meth.walk(callback, chain)
 
 
+class Pointer(Node, Registered):
+    """A pointer type with no known associated structure/union."""
+    def __init__(self, name,
+                 gtype_name=None,
+                 get_type=None,
+                 c_symbol_prefix=None):
+        assert gtype_name is not None
+        assert get_type is not None
+        Node.__init__(self, name)
+        Registered.__init__(self, gtype_name, get_type)
+        if get_type is not None:
+            assert c_symbol_prefix is not None
+        self.c_symbol_prefix = c_symbol_prefix
+        self.constructors = []
+        self.methods = []
+        self.static_methods = []
+
+    def _walk(self, callback, chain):
+        for ctor in self.constructors:
+            ctor.walk(callback, chain)
+        for meth in self.methods:
+            meth.walk(callback, chain)
+        for meth in self.static_methods:
+            meth.walk(callback, chain)
+
+
 class Signal(Callable):
 
     def __init__(self, name, retval, parameters, when=None,
