@@ -34,7 +34,7 @@ from .sourcescanner import (
     CSYMBOL_TYPE_FUNCTION, CSYMBOL_TYPE_FUNCTION_MACRO, CSYMBOL_TYPE_TYPEDEF, CSYMBOL_TYPE_STRUCT,
     CSYMBOL_TYPE_ENUM, CSYMBOL_TYPE_UNION, CSYMBOL_TYPE_OBJECT,
     CSYMBOL_TYPE_MEMBER, CSYMBOL_TYPE_ELLIPSIS, CSYMBOL_TYPE_CONST,
-    TYPE_QUALIFIER_CONST, TYPE_QUALIFIER_VOLATILE)
+    TYPE_QUALIFIER_CONST, TYPE_QUALIFIER_VOLATILE, FUNCTION_INLINE)
 
 
 class TransformerException(Exception):
@@ -451,7 +451,10 @@ raise ValueError."""
         parameters = list(self._create_parameters(symbol, symbol.base_type))
         return_ = self._create_return(symbol.base_type.base_type)
         name = self._strip_symbol(symbol)
+
         func = ast.Function(name, return_, parameters, False, symbol.ident)
+        if symbol.base_type.base_type.function_specifier & FUNCTION_INLINE:
+            func.is_inline = True
         func.add_symbol_reference(symbol)
         return func
 
@@ -1032,8 +1035,10 @@ Note that type resolution may not succeed."""
         while isinstance(typenode, ast.Alias):
             if typenode.target.target_giname is not None:
                 typenode = self.lookup_giname(typenode.target.target_giname)
-            elif typenode.target.target_fundamental is not None:
-                typenode = typenode.target
             else:
-                break
+                # This can happen when target_fundamental is "<array>"
+                try:
+                    typenode = ast.type_names[typenode.target.target_fundamental]
+                except KeyError:
+                    break
         return typenode
